package cz.frantovo.hrisniciSpameri;

import cz.frantovo.hrisniciSpameri.dao.CilDAO;
import cz.frantovo.hrisniciSpameri.dao.SouhrnDAO;
import cz.frantovo.hrisniciSpameri.dto.Cil;
import cz.frantovo.hrisniciSpameri.dto.Soucet;
import cz.frantovo.hrisniciSpameri.dto.Souhrn;
import java.io.Serializable;
import java.sql.Date;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Collection;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Beana zprostředkovávající denní souhrn hříšníků.
 * @author fiki
 */
public class DenniSouhrn implements Serializable {

    private static final long serialVersionUID = 7932392366943861342L;
    private static final Logger log = Logger.getLogger(DenniSouhrn.class.getName());
    private static String formatDataVzor = "yyyy-MM-dd";
    public DateFormat formatData = new SimpleDateFormat(formatDataVzor);
    private Date zacatek;
    private Date konec;
    private int cil;

    public String getZacatekString() {
        return formatData.format(getZacatek());
    }

    public Date getZacatek() {
        if (zacatek == null) {
            nastavVychoziZacatek();
        }
        return zacatek;
    }

    public void setZacatek(Date zacatek) {
        if (zacatek == null) {
            nastavVychoziZacatek();
        } else {
            this.zacatek = zacatek;
        }
        log.log(Level.FINE, "Začátek nastaven na: " + formatData.format(zacatek));
    }

    public void setZacatekString(String zacatek) {
        if (zacatek == null) {
            nastavVychoziZacatek();
        } else {
            try {
                this.zacatek = new Date(formatData.parse(zacatek).getTime());
            } catch (ParseException e) {
                log.log(Level.FINE, "Chybně zadané datum – začátek: " + zacatek, e);
                nastavVychoziZacatek();
            }
        }
    }

    public Date getKonec() {
        if (konec == null) {
            nastavVychoziKonec();
        }
        return konec;
    }

    public void setKonec(Date konec) {
        if (konec == null) {
            nastavVychoziKonec();
        } else {
            this.konec = konec;
        }
        log.log(Level.FINE, "Konec nastaven na: " + formatData.format(zacatek));
    }

    public void setKonecString(String konec) {
        if (konec == null) {
            nastavVychoziKonec();
        } else {
            try {
                this.konec = new Date(formatData.parse(konec).getTime());
            } catch (Exception e) {
                nastavVychoziKonec();
            }
        }
    }

    public String getKonecString() {
        return formatData.format(getKonec());
    }

    public void setCilString(String cil) {
        try {
            this.cil = Integer.parseInt(cil);
        } catch (Exception e) {
            this.cil = 0;
        }
    }

    public void setCil(int cil) {
        this.cil = cil;
    }

    public String getCilString() {
        return String.valueOf(cil);
    }

    public int getCil() {
        return cil;
    }

    private void nastavVychoziZacatek() {
        log.log(Level.FINE, "Nastavuji začátek na výchozí hodnotu.");
        setZacatek(getVychoziZacatek());
    }

    private void nastavVychoziKonec() {
        log.log(Level.FINE, "Nastavuji konec na výchozí hodnotu.");
        setKonec(getVychoziKonec());
    }

    /**
     * @return Denní souhrny útoků –
     * obsahuje IP adresy a domény útočníků,
     * datum je den poslendího útoku daného útočníka na daný cíl.
     */
    public Collection<Souhrn> getSouhrn() {
        Collection<Souhrn> souhrn = new SouhrnDAO().getSouhrn(zacatek, konec, cil);

        /** ne všechno musíme posílat do JSP vrstvy */
        for (Souhrn s : souhrn) {
            filtrujCil(s.getCil());
        }

        return souhrn;
    }

    /**
     * @return Vyšší stupeň agregace než souhrn –
     * obsahuje jen součty útoků pro daný den a daný cíl,
     * neobsahuje podrobnosti o útočnících (ip adresa, doména).
     */
    public Map<Cil, Collection<Soucet>> getSoucty() {
        Map<Cil, Collection<Soucet>> soucty = new SouhrnDAO().getSoucty(zacatek, konec, cil);

        /** ne všechno musíme posílat do prezentační vrstvy */
        for (Collection<Soucet> ss : soucty.values()) {
            for (Soucet s : ss) {
                filtrujCil(s.getCil());
            }
        }

        return soucty;
    }

    /**
     * @return Seznam všech cílů. Ale bez těchto hodnot: databáze a selekt.
     */
    public Collection<Cil> getSeznamCilu() {
        Collection<Cil> cile = new CilDAO().getCile();
        /** ne všechno musíme posílat do prezentační vrstvy */
        filtrujCile(cile);
        return cile;
    }

    /**
     * Vymaže u cílů hodnoty „databáze“ a „selekt“ – ktreré v prezentační vrstvě většinou nepotřebujeme
     * @param cile kolekce, který bude upravena
     */
    private static void filtrujCile(Collection<Cil> cile) {
        for (Cil c : cile) {
            filtrujCil(c);
        }
    }

    /**
     * Vymaže u cíle hodnoty „databáze“ a „selekt“ – ktreré v prezentační vrstvě většinou nepotřebujeme
     * @param c cíl, který bude upraven
     */
    private static void filtrujCil(Cil c) {
        c.setDatabaze(null);
        c.setSelekt(null);
    }

    /**
     * @return Výchozí začátek je před měsícem
     */
    private static Date getVychoziZacatek() {
        Calendar c = Calendar.getInstance();
        c.add(Calendar.MONTH, -1);
        return new Date(c.getTimeInMillis());
    }

    /**
     * @return  Výchozí konec je současnost
     */
    private static Date getVychoziKonec() {
        return new Date(System.currentTimeMillis());
    }
}
